import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline, confirm


class GitgraberRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="hisxo/gitGraber",
            install={"pip": "requirements.txt"},
            description="Search and find sensitive data in real time for GitHub"
        )

    def run(self):
        os.chdir(self.full_path)
        wordlists_path = os.path.join(self.full_path, "wordlists")
        set_readline([])
        user_query = input("\nEnter a search query: ").strip()
        set_readline([
            f for f in os.listdir(os.path.join(self.full_path, "wordlists"))
            if os.path.isfile(
                os.path.join(os.path.join(self.full_path, "wordlists"), f))
        ])
        user_keywords = input(
            "\nEnter a keywords path [default=keywords.txt]: ").strip()
        if not user_keywords:
            user_keywords = "keywords.txt"
        keywords_path = os.path.join(wordlists_path, user_keywords)
        if confirm("\nDo you want to add a GitHub token?"):
            github_token = input("\nEnter a GitHub token: ").strip()
            os.environ["GITHUB_TOKENS"] = f"['{github_token}']"
        return os.system(
            f"python3 gitGraber.py -k {keywords_path} -q {user_query}")


gitgraber = GitgraberRepo()



# pylint: disable=line-too-long
import os
from shutil import which

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline
from fsociety.core.hosts import get_hosts, add_host, InvalidHost

premade_args = {
    "simple":
    "{host}",
    "common_ports":
    "-F {host}",
    "all_ports":
    "-p- {host}",
    "detect_os":
    "-A {host}",
    "tcp_syn_scan":
    "-sS {host}",
    "tcp_connect":
    "-sT {host}",
    "nse_standard":
    "-sV -sC {host}",
    "vuln_scan":
    "-Pn --script vuln {host}",
    "google_malware":
    "-p80 --script http-google-malware {host}",
    "argressive_scan":
    "-A -T4 {host}",
    "detect_web_app":
    "--script=http-enum {host}",
    "subdomain_enumaration":
    "-sn --script hostmap-crtsh {host}",
    "heartbleed_test":
    "-sV -p 443 --script=ssl-heartbleed {host}",
    "slowloris":
    "-max-parallelism 800 -Pn --script http-slowloris --script-args http-slowloris.runforever=true {host}"
}


class NmapRepo(GitHubRepo):
    def __init__(self):
        super().__init__(path="nmap/nmap",
                         install={
                             "linux": "sudo apt-get install nmap",
                             "brew": "install nmap"
                         },
                         description="the Network Mapper")

    def installed(self):
        return which("nmap")

    def install(self):
        super().install(clone=False)

    def run(self):
        hosts = get_hosts()
        set_readline(hosts)
        host = input("\nEnter a host: ").strip()
        if not host:
            raise InvalidHost
        if host not in hosts:
            add_host(host)
        longest_key = max([len(key) for key in premade_args]) + 2
        print("\nName".ljust(longest_key) + " | Args")
        for name, args in premade_args.items():
            print(f"{name.ljust(longest_key)}: {args.format(host=host)}")
        set_readline(premade_args.keys())
        selected = input("\nMake a selection: ")
        if selected and selected in premade_args.keys():
            args = premade_args.get(selected).format(host=host)
            return os.system(f"nmap {args}")
        return self.run()


nmap = NmapRepo()



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline


class Cr3dov3rRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="D4Vinci/Cr3dOv3r",
            install={"pip": "requirements.txt"},
            description="Your best friend in credential reuse attacks")

    def run(self):
        os.chdir(self.full_path)
        set_readline([])
        user_email = input("\nEnter a email: ").strip()
        return os.system(f"python3 Cr3d0v3r.py {user_email}")


cr3dov3r = Cr3dov3rRepo()



# pylint: disable=too-many-branches,line-too-long
import os
from shutil import rmtree, which
from abc import ABCMeta, abstractmethod
from collections.abc import Iterable

from git import Repo, RemoteProgress
from rich.progress import BarColumn, Progress
from rich.table import Table

from fsociety.core.config import INSTALL_DIR, get_config
from fsociety.core.menu import confirm
from fsociety.console import console

config = get_config()


def print_pip_deps(packages):
    requirements = list()
    if isinstance(packages, str) and os.path.exists(packages):
        with open(packages, "r") as requirements_file:
            for line in requirements_file:
                if line.strip():
                    requirements.append(line)
    elif isinstance(packages, Iterable):
        requirements = packages
    else:
        raise ValueError
    table = Table("Packages", title="Pip Dependencies")
    for req in requirements:
        table.add_row(req)
    console.print()
    console.print(table)


class InstallError(Exception):
    pass


class CloneError(Exception):
    pass


class GitProgress(RemoteProgress):
    def __init__(self):
        super().__init__()
        self.progress = Progress(
            "[progress.description]{task.description}",
            BarColumn(None),
            "[progress.percentage]{task.percentage:>3.0f}%",
            "[progress.filesize]{task.fields[msg]}",
        )
        self.current_opcode = None
        self.task = None

    def update(self, opcode, count, max_value, msg=None):
        opcode_strs = {
            self.COUNTING: "Counting",
            self.COMPRESSING: "Compressing",
            self.WRITING: "Writing",
            self.RECEIVING: "Receiving",
            self.RESOLVING: "Resolving",
            self.FINDING_SOURCES: "Finding sources",
            self.CHECKING_OUT: "Checking out",
        }
        stage, real_opcode = opcode & self.STAGE_MASK, opcode & self.OP_MASK

        try:
            count = int(count)
            max_value = int(max_value)
        except ValueError:
            return

        if self.current_opcode != real_opcode:
            if self.task:
                self.progress.update(self.task, total=1, completed=1, msg="")
            self.current_opcode = real_opcode
            self.task = self.progress.add_task(
                opcode_strs[real_opcode].ljust(15), msg=""
            )

        if stage & self.BEGIN:
            self.progress.start()
        if stage & self.END:
            self.progress.stop()
        self.progress.update(self.task, msg=msg or "", total=max_value, completed=count)


class GitHubRepo(metaclass=ABCMeta):
    def __init__(
        self,
        path="fsociety-team/fsociety",
        install="pip install -e .",
        description=None,
    ):
        self.path = path
        self.name = self.path.split("/")[-1]
        self.install_options = install
        self.full_path = os.path.join(INSTALL_DIR, self.name)
        self.description = description

    def __str__(self):
        return self.name.lower().replace("-", "_")

    def clone(self, overwrite=False):
        if os.path.exists(self.full_path):
            if not overwrite:
                repo = Repo(self.full_path)
                repo.remotes.origin.pull()
                return self.full_path
            rmtree(self.full_path)
        url = f"https://github.com/{self.path}"
        if config.getboolean("fsociety", "ssh_clone"):
            url = f"git@github.com:{self.path}.git"
        Repo.clone_from(url, self.full_path, progress=GitProgress())
        if not os.path.exists(self.full_path):
            raise CloneError(f"{self.full_path} not found")
        return self.full_path

    def install(self, no_confirm=False, clone=True):
        if no_confirm or not confirm(
            f"\nDo you want to install https://github.com/{self.path}?"
        ):
            print("Cancelled")
            return
        if clone:
            self.clone()
        if self.install_options:
            if clone:
                os.chdir(self.full_path)
            else:
                os.chdir(INSTALL_DIR)
            install = self.install_options

            if isinstance(install, dict):
                if "pip" in install.keys():
                    packages = install.get("pip")
                    if isinstance(packages, list):
                        message = "Do you want to install these packages?"
                        packages_str = " ".join(packages)
                        command = f"pip install {packages_str}"
                    elif isinstance(packages, str):
                        requirements_txt = os.path.join(
                            self.full_path, "requirements.txt"
                        )
                        message = f"Do you want to install these packages from {requirements_txt}?"
                        command = f"pip install -r {requirements_txt}"

                    print_pip_deps(packages)
                    if not confirm(message):
                        raise InstallError("User Cancelled")
                elif (
                    config.get("fsociety", "os") == "macos"
                    and "brew" in install.keys()
                    and which("brew")
                ):
                    brew_opts = install.get("brew")
                    command = f"brew {brew_opts}"
                elif (
                    "linux" in install.keys()
                    or "windows" in install.keys()
                    or "macs" in install.keys()
                ):
                    command = install.get(
                        config.get("fsociety", "os"), install.get("linux")
                    )
            else:
                command = install

            os.system(command)

    def installed(self):
        return os.path.exists(self.full_path)

    @abstractmethod
    def run(self):
        pass



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline
from fsociety.core.config import INSTALL_DIR


class S3scannerRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="sa7mon/S3Scanner",
            install={"pip": "requirements.txt"},
            description="A tool to find open S3 buckets and dump their contents")

    def run(self):
        os.chdir(self.full_path)
        set_readline([])
        user_domains = input("\nEnter one or more domains: ").strip()
        txt_path = os.path.join(INSTALL_DIR, "s3_domains.txt")
        with open(txt_path, "w") as domains_file:
            for domain in user_domains.split():
                domains_file.write(domain)
        return os.system(f"python3 s3scanner.py {txt_path}")


s3scanner = S3scannerRepo()



# Core
from fsociety.core.menu import tools_cli

from .sqlmap import sqlmap
from .striker import striker
from .sublist3r import sublist3r
from .sherlock import sherlock
from .s3scanner import s3scanner
from .gitgraber import gitgraber
from .hydrarecon import hydrarecon

__tools__ = [sqlmap, striker, sublist3r,
             sherlock, s3scanner, gitgraber, hydrarecon]


def cli():
    tools_cli(__name__, __tools__)



import os.path

from fsociety.core.config import INSTALL_DIR, get_config

config = get_config()

full_path = os.path.join(INSTALL_DIR, config.get("fsociety", "usernames_file"))


def get_usernames():
    try:
        with open(full_path, "r") as usernamefile:
            return [username.strip() for username in usernamefile]
    except FileNotFoundError:
        return list()


def add_username(username):
    with open(full_path, "a") as usernamefile:
        usernamefile.write(f"\n{username}")



# pylint: disable=unused-import,broad-except,inconsistent-return-statements
import os
import shutil

from rich.text import Text
from rich.table import Table
from rich.style import Style
from rich import box

from fsociety.console import console
from fsociety.core.config import INSTALL_DIR

BACK_COMMANDS = ["back", "return"]


class CommandCompleter:
    def __init__(self, options):
        self.options = sorted(options)
        self.matches = list()

    def complete(self, text, state):
        response = None
        if state == 0:
            if text:
                self.matches = [s for s in self.options if s and s.startswith(text)]
            else:
                self.matches = self.options[:]
        try:
            response = self.matches[state]
        except IndexError:
            response = None
        return response


def set_readline(items):
    try:
        import readline
    except ImportError:
        pass
    else:
        import rlcompleter

        if isinstance(items, list):
            readline.set_completer(CommandCompleter(items).complete)
        elif isinstance(items, dict):
            readline.set_completer(CommandCompleter(items.keys()).complete)
        else:
            readline.set_completer(CommandCompleter(list(items)).complete)
        readline.parse_and_bind("tab: complete")


def clear_screen():
    os.system("cls" if os.name == "nt" else "clear")


def format_tools(tools):
    cutoff = 5
    etc = False
    if len(tools) > cutoff:
        tools = tools[:cutoff]
        etc = True
    res = "".join([f"\n{str(tool)}" for tool in tools])
    if etc:
        res += "\n..."
    return res


def module_name(module):
    return module.__name__.split(".")[-1]


def prompt(path="", base_path="~"):
    encoded_path = os.path.join(base_path, path, "")
    return f"\nfsociety {encoded_path}# "


def input_wait():
    input("\nPress [ENTER] to continue... ")


def tools_cli(name, tools, links=True):
    table = Table(box=box.HEAVY_HEAD)
    table.add_column("Name", style="red", no_wrap=True)
    table.add_column("Description", style="magenta")
    if links:
        table.add_column("Link", no_wrap=True)

    tools_dict = dict()
    for tool in tools:
        tools_dict[str(tool)] = tool
        args = [str(tool), tool.description]
        if links:
            text_link = Text(f"{tool.path}")
            text_link.stylize(Style(link=f"https://github.com/{tool.path}"))
            args.append(text_link)
        table.add_row(*args)

    console.print(table)
    console.print("back", style="command")
    set_readline(list(tools_dict.keys()) + BACK_COMMANDS)
    selected_tool = input(prompt(name.split(".")[-2])).strip()
    if selected_tool not in tools_dict.keys():
        if selected_tool in BACK_COMMANDS:
            return
        console.print("Invalid Command", style="bold yellow")
        return tools_cli(name, tools, links)
    tool = tools_dict.get(selected_tool)
    if hasattr(tool, "install") and not tool.installed():
        tool.install()
    try:
        response = tool.run()
        if response and response > 0 and response != 256:
            console.print(
                f"{selected_tool} returned a non-zero exit code", style="bold red"
            )
            if hasattr(tool, "install") and confirm("Do you want to reinstall?"):
                os.chdir(INSTALL_DIR)
                shutil.rmtree(tool.full_path)
                tool.install()
    except KeyboardInterrupt:
        return

    return input_wait()


def confirm(message="Do you want to?"):
    response = input(f"{message} (y/n): ").lower()
    if response:
        return response[0] == "y"
    return False



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline, confirm
from fsociety.core.config import INSTALL_DIR


class HydrareconRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="aufzayed/HydraRecon",
            install={"pip": "requirements.txt"},
            description="Simple recon tool",
        )

    def run(self):
        os.chdir(self.full_path)
        set_readline([])
        user_domain = input("\nEnter a domain to scan: ").strip()
        arg = "--basic"
        if confirm("\nDo you want to crawl? [default=No]"):
            arg = "--crawl"
        return os.system(
            f"python3 hydrarecon.py -d {user_domain} -o {INSTALL_DIR} {arg}"
        )


hydrarecon = HydrareconRepo()



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline


class StrikerRepo(GitHubRepo):
    def __init__(self):
        super().__init__(path="s0md3v/Striker",
                         install={"pip": "requirements.txt"},
                         description="Recon & Vulnerability Scanning Suite")

    def run(self):
        os.chdir(self.full_path)
        set_readline([])
        user_domain = input("\nEnter a domain to scan: ").strip()
        return os.system(f"python3 striker.py {user_domain}")


striker = StrikerRepo()



from rich.console import Console
from rich.theme import Theme
from rich.traceback import install

# Install Traceback
install()

# Console Setup
fsociety_theme = Theme({"command": "black on white", "warning": "bold yellow",})
console = Console(theme=fsociety_theme)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
import sys

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = "fsociety"
DESCRIPTION = "A Modular Penetration Testing Framework"
URL = "https://fsociety.dev/"
PROJECT_URLS = {
    "Packages": "https://github.com/fsociety-team/fsociety/blob/master/PACKAGES.md",
    "Changelog": "https://github.com/fsociety-team/fsociety/blob/master/CHANGELOG.md",
    "Funding": "https://github.com/sponsors/thehappydinoa",
    "Tracker": "https://github.com/fsociety-team/fsociety/issues",
    "Source": "https://github.com/fsociety-team/fsociety",
}
EMAIL = "contact@fsociety.dev"
AUTHOR = "fsociety-team"
REQUIRES_PYTHON = ">=3.7.0"
VERSION = None

# Required Packages
REQUIRED = ["gitpython", "rich>=9.2.0", "requests"]

# Optional Packages
EXTRAS = {"dev": ["wheel", "pylint", "autopep8", "twine", "mypy", "flake8"]}

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
        long_description = "\n" + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, "__version__.py")) as f:
        exec(f.read(), about)
else:
    about["__version__"] = VERSION


class TagCommand(Command):
    """Support setup.py push_tag."""

    description = "Push latest version as tag."
    user_options = []

    @staticmethod
    def status(s):
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        self.status("Pushing git tags…")
        os.system("git tag v{0}".format(about["__version__"]))
        os.system("git push --tags")

        sys.exit()


setup(
    name=NAME,
    version=about["__version__"],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    project_urls=PROJECT_URLS,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    entry_points={"console_scripts": ["fsociety=fsociety:cli"],},
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license="MIT",
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Topic :: Security",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    # python setup.py upload
    cmdclass={"push_tag": TagCommand},
)
